#include <stdint.h>

static uint64_t load_3(const unsigned char *in) {
	uint64_t result;
	result = (uint64_t) in[0];
	result |= ((uint64_t) in[1]) << 8;
	result |= ((uint64_t) in[2]) << 16;
	return result;
}

static uint64_t load_4(const unsigned char *in) {
	uint64_t result;
	result = (uint64_t) in[0];
	result |= ((uint64_t) in[1]) << 8;
	result |= ((uint64_t) in[2]) << 16;
	result |= ((uint64_t) in[3]) << 24;
	return result;
}

void fe_frombytes(uint32_t h[10], const unsigned char *s) {
	int64_t h0 = load_4(s);
	int64_t h1 = load_3(s + 4) << 6;
	int64_t h2 = load_3(s + 7) << 5;
	int64_t h3 = load_3(s + 10) << 3;
	int64_t h4 = load_3(s + 13) << 2;
	int64_t h5 = load_4(s + 16);
	int64_t h6 = load_3(s + 20) << 7;
	int64_t h7 = load_3(s + 23) << 5;
	int64_t h8 = load_3(s + 26) << 4;
	int64_t h9 = (load_3(s + 29) & 8388607) << 2;
	int64_t carry0;
	int64_t carry1;
	int64_t carry2;
	int64_t carry3;
	int64_t carry4;
	int64_t carry5;
	int64_t carry6;
	int64_t carry7;
	int64_t carry8;
	int64_t carry9;

	carry9 = (h9 + (int64_t) (1<<24)) >> 25; h0 += carry9 * 19; h9 -= carry9 << 25;
	carry1 = (h1 + (int64_t) (1<<24)) >> 25; h2 += carry1; h1 -= carry1 << 25;
	carry3 = (h3 + (int64_t) (1<<24)) >> 25; h4 += carry3; h3 -= carry3 << 25;
	carry5 = (h5 + (int64_t) (1<<24)) >> 25; h6 += carry5; h5 -= carry5 << 25;
	carry7 = (h7 + (int64_t) (1<<24)) >> 25; h8 += carry7; h7 -= carry7 << 25;

	carry0 = (h0 + (int64_t) (1<<25)) >> 26; h1 += carry0; h0 -= carry0 << 26;
	carry2 = (h2 + (int64_t) (1<<25)) >> 26; h3 += carry2; h2 -= carry2 << 26;
	carry4 = (h4 + (int64_t) (1<<25)) >> 26; h5 += carry4; h4 -= carry4 << 26;
	carry6 = (h6 + (int64_t) (1<<25)) >> 26; h7 += carry6; h6 -= carry6 << 26;
	carry8 = (h8 + (int64_t) (1<<25)) >> 26; h9 += carry8; h8 -= carry8 << 26;

	h[0] = h0;
	h[1] = h1;
	h[2] = h2;
	h[3] = h3;
	h[4] = h4;
	h[5] = h5;
	h[6] = h6;
	h[7] = h7;
	h[8] = h8;
	h[9] = h9;
}
